/*

 Copyright 2013, SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.javafmi.wrapper.FmuProxy;

public class VariableWriter {

    private final Map<String, Method> setterMethodsByType;
    private FmuProxy fmuProxy;

    public VariableWriter(FmuProxy fmuProxy) {
        this.fmuProxy = fmuProxy;
        setterMethodsByType = new HashMap<>();
        try {
            setterMethodsByType.put("Real", FmuProxy.class.getMethod("setReal", Integer.class, Double.class));
            setterMethodsByType.put("Integer", FmuProxy.class.getMethod("setInteger", Integer.class, Integer.class));
            setterMethodsByType.put("Boolean", FmuProxy.class.getMethod("setBoolean", Integer.class, Boolean.class));
            setterMethodsByType.put("String", FmuProxy.class.getMethod("setString", Integer.class, String.class));
        } catch (NoSuchMethodException | SecurityException ex) {
            Logger.getLogger(VariableWriter.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public Status write(String name, Object value) {
        Integer valueReference = fmuProxy.getModelDescription().getScalarVariable(name).getValueReference();
        try {
            return (Status) setterMethodsByType.get(fmuProxy.getScalarVariable(name).getTypeName()).invoke(fmuProxy, valueReference, castToDefinedType(name, value));
        } catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException ex) {
            Logger.getLogger(VariableWriter.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    private Object castToDefinedType(String name, Object value) {
        return (isReal(name)) ? getDoubleValue(value) : value;
    }

    private boolean isReal(String name) {
        return fmuProxy.getScalarVariable(name).getTypeName().equals("Real");
    }

    private double getDoubleValue(Object value) {
        return Double.parseDouble(String.valueOf(value));
    }
}
